'''Curses emulation with PyGame.'''

import data
import os
import pygame
import sys
import time



COLOR_BLACK   = 0
COLOR_BLUE  = 1
COLOR_GREEN   = 2
COLOR_CYAN  = 3
COLOR_RED   = 4
COLOR_MAGENTA = 5
COLOR_YELLOW  = 6
COLOR_WHITE   = 7

A_BOLD = 8

KEY_UP  = 257
KEY_DOWN  = 258
KEY_LEFT  = 259
KEY_RIGHT = 260


KEY_F1  = pygame.K_F1
KEY_F2  = pygame.K_F2
KEY_F3  = pygame.K_F3
KEY_F4  = pygame.K_F4
KEY_F5  = pygame.K_F5
KEY_F6  = pygame.K_F6
KEY_F7  = pygame.K_F7
KEY_F8  = pygame.K_F8
KEY_F9  = pygame.K_F9
KEY_F10 = pygame.K_F10
KEY_F11 = pygame.K_F11
KEY_F12 = pygame.K_F12



def wrapper(method):
  pygame.init()
  
  method(StdScr(24, 80))



def delay_output(ms):
  time.sleep(ms / 1000.0)


class StdScr:
  COLORS = [(  0,   0,   0), ( 32,  32, 192), (  0, 192,   0), (  0, 192, 192),
            (192,   0,   0), (192,   0, 192), (255, 128,   0), (192, 192, 192),
            (128, 128, 128), (128, 128, 255), ( 96, 255,  96), ( 96, 255, 255),
            (255,  96,  96), (255,  96, 255), (255, 255,  96), (255, 255, 255)]
  
  def __init__(self, height, width):
     
    pygame.key.set_repeat(250, 100)

    self._keyBuffer = []
    
    self._id   = 0
    self._height = height
    self._width  = width
    
    font = pygame.font.match_font('Lucida Console')

    if font is None or font.lower().find('lucida') == -1:
      # Not found or found something weird
      self._font = data.loadFont('FreeMono.ttf', 16)

      (x, y) = self._font.size('@')
    
      y *= 2
      y /= 3
      
      self._dy = -y / 3
    else:
      self._font = pygame.font.SysFont('Lucida Console', 16)

      (x, y) = self._font.size('@')
    
      self._dy = 0
    
#    name = os.path.join("..", self.DATADIR, 'Icon.gif')
#    pygame.display.set_icon(pygame.image.load(name))
    
    self._screen = pygame.display.set_mode((x * width, y * height))
    self._x_size = x
    self._y_size = y
    
    pygame.display.set_caption('RegEx Express')
    
    self._chars = [[(' ', (0, 0, 0))
            for y in range(height)]
            for x in range(width)]
    
    self._boxes = {0: None}
  
  def _char(self, x, y):
    return self._chars[x][y][0]
  
  def _color(self, x, y):
    return self._chars[x][y][1]
  
  def _rect(self, x, y):
    return pygame.Rect(x * self._x_size, y * self._y_size + self._dy,
               self._x_size, self._y_size)
  
  def boxes(self):
    return self._boxes
  
  def id(self):
    return 0
  
  def x(self):
    return 0
  
  def y(self):
    return 0
    
  def move(self, y, x):
    pass
  
  def width(self):
    return self._width
  
  def height(self):
    return self._height
    
  def getmaxyx(self):
    return self.height(), self.width()
  
  def screen(self):
    return self._screen
  
  def font(self):
    return self._font
  
  def subwin(self, height, width, y, x):
    self._id += 1
    self._boxes[self._id] = None
    
    return SubWin(self, self._id, height, width, y, x)

  def addch (self, y, x, chrid, color = COLOR_WHITE):
    if (type(chrid) is int):
      self.addstr(y,x,chr(chrid),color)
    else:
      self.addstr(y,x,chrid,color)
    
  def addstr(self, y, x, text, color = COLOR_WHITE):
    dx = 0
    
    for c in text:
      self._chars[x + dx][y] = (c, self.COLORS[int(color)])
      dx += 1
  
  def refresh(self):
    self.screen().fill((0, 0, 0))
    
    for x in range(self.width()):
      for y in range(self.height()):
        char  = self._char(x, y)
        color = self._color(x, y)
        text  = self.font().render(char, True, color)
        self.screen().blit(text, self._rect(x, y))
    
    for box in self.boxes().values():
      if box is None:
        continue
      
      (x, y, width, height) = box
      
      px = x * self._x_size + self._x_size / 2 
      py = y * self._y_size + self._y_size / 2 
      pw = width * self._x_size - self._x_size  + 1
      ph = height * self._y_size - self._y_size + 1
      
      pygame.draw.rect(self.screen(), self.COLORS[COLOR_WHITE],
               pygame.Rect(px, py, pw, ph), 1)
      
    pygame.display.flip()
  
  def clear(self):
    self._boxes = {0: None}
    
    for x in range(self.width()):
      for y in range(self.height()):
        self.addstr(y, x, ' ', COLOR_BLACK)
  
  def erase(self):
    self.clear()
  
  def box(self):
    self.add_box(self)
  
  def add_box(self, window):
    self._boxes[window.id()] = (window.x(), window.y(),
                  window.width(), window.height())
  
  def del_box(self, window):
    self._boxes[window.id()] = None
  
  def timeout(self, time):
    pass
    
  def attrset(self, color):
    pass
    
  def nodelay(self, foo):
    pass
  
  def vline(self, num1, num2 = 0, num3 = 0, num4 = 0, num5 = 0):
    pass

  def hline(self, num1, num2 = 0, num3 = 0, num4 = 0, num5 = 0):
    pass
  
  def getch(self):
    key = None

#    while key is None:
    if (True):
      for event in pygame.event.get():
        if event.type == pygame.QUIT:
          return 27 # ESC
        elif event.type == pygame.KEYDOWN:
          key = 0
          
          if event.key == pygame.K_UP:
            key = KEY_UP
          elif event.key == pygame.K_DOWN:
            key = KEY_DOWN
          elif event.key == pygame.K_LEFT:
            key = KEY_LEFT
          elif event.key == pygame.K_RIGHT:
            key = KEY_RIGHT
          elif pygame.K_F1 <= event.key <= pygame.K_F12:
            key = event.key
          elif len(event.unicode) >= 1:
            key = ord(event.unicode)
          else:
            key = event.key
            print "Pressed unhandled key " + str(event.key)
            
          self._keyBuffer.append(key)
        
    
    if (len(self._keyBuffer) > 0):
      return self._keyBuffer.pop()
    else:
      return 0


class SubWin:
  def __init__(self, stdscr, id, height, width, y, x):
    self._stdscr = stdscr
    self._id   = id
    self._x    = x
    self._y    = y
    self._width  = width
    self._height = height
    
  def stdscr(self):
    return self._stdscr
  
  def id(self):
    return self._id
  
  def x(self):
    return self._x
  
  def y(self):
    return self._y
  
  def move(self, y, x):
    self._x = x
    self._y = y
    
  def width(self):
    return self._width
  
  def height(self):
    return self._height

  def getmaxyx(self):
    return self.height(), self.width()
  
  def clear(self):
    self.stdscr().del_box(self)

    for x in range(self.width()):
      for y in range(self.height()):
        self.addstr(y, x, ' ', COLOR_BLACK)
  
  def erase(self):
    self.clear()
  
  def box(self):
    self.stdscr().add_box(self)
  
  def addch(self, y, x, chrid, color = COLOR_WHITE):
    self.stdscr().addch(y + self.y(), x + self.x(), chrid, color)
  
  def addstr(self, y, x, text, color = COLOR_WHITE):
    self.stdscr().addstr(y + self.y(), x + self.x(), text, color)
  
  def syncup(self): pass

  def refresh(self): 
    self.stdscr().refresh()
    

def curs_set(foo):   pass
def flushinp():    pass
def init_pair(*foo): pass
def color_pair(foo): return foo
def has_colors():  return True
def nl():      pass
def noecho():    pass
def napms(foo):    pass

def newwin(height, width, uly, ulx):
  return stdscr.subwin(height, width, uly, ulx)
  
  
COLS =  ACS_HLINE = 80
LINES = ACS_VLINE = 24
